/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.perseus.cache;

import org.objectweb.fractal.adl.Parser;
import org.objectweb.fractal.adl.Launcher;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.fractal.api.Component;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class FractalHelper {

	private final static String FRACTAL_PROVIDER = "fractal.provider";
	private final static String DEFAULT_FRACTAL_PROVIDER
			= "org.objectweb.fractal.julia.Julia";

	private final static String JULIA_LOADER = "julia.loader";
	private final static String DEFAULT_JULIA_LOADER
			= "org.objectweb.fractal.julia.loader.DynamicLoader";

	private final static String JULIA_CONFIG = "julia.config";
	private final static String DEFAULT_JULIA_CONFIG = "julia.cfg";

	private static Parser parser;

	/**
	 * Retrieve the unique julia parser instance. If the parser does not already
	 * exist, it is created.
	 * @throws java.lang.Exception if an error occur during the creationg of the parser.
	 */
	private static Parser getParser() throws Exception {
		if (parser == null) {
			synchronized(FractalHelper.class) {
				if (parser == null) {
					System.setProperty(FRACTAL_PROVIDER, DEFAULT_FRACTAL_PROVIDER);
					System.setProperty(JULIA_LOADER, DEFAULT_JULIA_LOADER);
					System.setProperty(JULIA_CONFIG, DEFAULT_JULIA_CONFIG);
					parser = Launcher.getBootstrapParser();
				}
			}
		}
		return parser;
	}


	/**
	 * Instanciate a component since the template name
	 * @param templateName is the name of the template to load and to
	 * instanciate
	 * @return the component instance
	 * @throws java.lang.Exception if an error occurs during the template loading or in
	 * the component instanciation.
	 */
	public static Component instanciate(String templateName) throws Exception {
		return instanciate(getParser().loadTemplate(templateName, true));
	}

	/**
	 * Instanciate a component since the template component
	 */
	public static Component instanciate(Component template) throws Exception {
		return Fractal.getFactory(template).newFcInstance();
	}

}
